import logging
import os

# OpenStack Ubuntu Testing repository source repo location
OSUT_ROOT = os.path.join(os.getenv('HOME'), 'tools',
                         'openstack-ubuntu-testing')

CURRENT_OS_DEV = "havana"

# Ubuntu -> Openstack release
OS_UBUNTU_VERSION_MAP = {
    'precise': 'essex',
    'quantal': 'folsom',
    'raring': 'grizzly',
    'saucy': 'havana',
    'trusty': 'icehouse',
    'utopic': 'juno',
    'vivid': 'kilo',
}

# Cloud Archive release -> Openstack version(s)
CA_OPENSTACK_VERSION_MAP = {
    'precise': ['folsom', 'grizzly', 'havana', 'icehouse'],
    'trusty': ['utopic', 'kilo']
}


def openstack_release(ubuntu_release):
    """
    Return the Openstack release shipped with a particular release of Ubuntu.

    :param ubuntu_release: Ubuntu release codename (eg, precise)
    """
    try:
        return OS_UBUNTU_VERSION_MAP[ubuntu_release]
    except KeyError:
        logging.error('Could not determine Openstack release for '\
                      'Ubuntu release %s' % ubuntu_release)
        return None


def cloud_archive_release(openstack_release):
    """
    Return the Ubuntu Release corresponding to the Cloud Archive pocket that
    provides a particular version of Openstack.

    :param openstack_release: Openstack release codename (eg, grizzly).
    """
    for k, v in CA_OPENSTACK_VERSION_MAP.iteritems():
        if openstack_release in v:
            return k
    logging.error('Could not determine CA release for OS release %s' %\
                  openstack_release)
    return None
