import os
import sys
import json
import yaml

from openstack_ubuntu_testing.env import inspectors


CLASS_PREFIXES = ['OSEnv']
REQUIRED_METHODS = ['do_inspection']


def import_module(module_name):
    __import__(module_name)
    return sys.modules[module_name]


def supports_inspection(module):
    for cl in dir(module):
        if True in [cl.startswith(prefix) for prefix in CLASS_PREFIXES]:
            _dir = dir(getattr(module, cl))
            if set(REQUIRED_METHODS).issubset(set(_dir)):
                return True
    return False


def load_inspectors_from_module(module):
    _inspectors = []
    for cl in dir(module):
        if True in [cl.startswith(prefix) for prefix in CLASS_PREFIXES]:
            _cl = getattr(module, cl)
            _inspectors.append(_cl())
    return _inspectors


class OSEnvironmentInspector(object):
    def __init__(self):
        self._data = {}
        self._inspectors = []
        self._load()

    def _load(self):
        for f in os.listdir(os.path.dirname(inspectors.__file__)):
            if f.startswith('env_') and f.endswith('.py'):
                f = f.split('.')[0]
                m = 'openstack_ubuntu_testing.env.inspectors.%s' % f
                mod = import_module(m)
                if supports_inspection(mod):
                    self._inspectors += load_inspectors_from_module(mod)

    def do_inspection(self):
        [self._data.update(insp.do_inspection(self._data))
         for insp in self._inspectors]

    @property
    def json(self):
        if not self._data:
            self.do_inspection()
        return json.dumps(self._data)

    @property
    def yaml(self):
        if not self._data:
            self.do_inspection()
        return yaml.dump(self._data)

    @property
    def shell(self):
        if not self._data:
            self.do_inspection()
        out = ''
        keys = sorted(self._data.iterkeys())
        for k in keys:
            out += 'export %s=%s\n' % (k.upper(), self._data[k])
        return out

    def tempest_config(self, openstack_release='havana'):
        pass
