import base

from keystoneclient.v2_0.client import Client as ksc

REQUIRED_HOSTS = [
    'nova', 'glance', 'keystone'
]


class OSEnvHostInspector(base.OSEnvInspectorBase):
    def __init__(self):
        self._envdata = {}
        self.ksc = None
        super(OSEnvHostInspector, self).__init__()

    def _get_ksc(self):
        try:
            creds = {
                'username': self._envdata['os_username'],
                'password': self._envdata['os_password'],
                'auth_url': self._envdata['os_auth_url'],
                'tenant_name': self._envdata['os_tenant_name'],
            }
        except KeyError:
            return False

        self.ksc = ksc(**creds)

        return True

    def os_admin_creds(self):
        url = 'http://%s:5000/v2.0/' % self._envdata['keystone_host']
        # TODO: Pull these values from keystone charm config
        creds = {
            'os_auth_url': url,
            'os_username': 'admin',
            'os_tenant_name': 'admin',
            'os_password': 'openstack',
            'admin_token': 'ubuntutesting',
        }

        self._envdata.update(creds)

        if not self.ksc and not self._get_ksc():
            return

        self._envdata['tenant_id'] = self.ksc.tenant_id

    def ec2_admin_creds(self):
        if not self.ksc and not self._get_ksc():
            return
        ec2_creds = self.ksc.ec2.create(self.ksc.user_id, self.ksc.tenant_id)
        ec2_url = self.ksc.service_catalog.url_for(
            service_type='ec2')
        ec2 = {
            'ec2_access_key': ec2_creds.access,
            'ec2_secret_key': ec2_creds.secret,
            'ec2_url': ec2_url,
        }
        self._envdata.update(ec2)

    def find_hosts(self):
        '''Locates all relevant openstack hosts in juju environment and
        ensures required hosts are present'''

        # TODO: Juju deployer find
        hosts = {
            'nova_host': self.find_service('nova-cloud-controller'),
            'glance_host': self.find_service('glance'),
            'keystone_host': self.find_service('keystone'),
            'nova_volume_host': self.find_service('nova-volume'),
            'cinder_host': self.find_service('cinder'),
            'neutron_host': self.find_service('neutron'),
            'quantum_host': self.find_service('quantum'),
            'horizon_host': self.find_service('horizon'),
            'ceph_host': self.find_service('ceph'),
            'swift_host': self.find_service('swift-proxy'),
            'nova_vmware': self.find_service('nova-vmware'),
        }

        # Handle neutron -> quantum rename for now.
        if hosts['neutron_host']:
            hosts['quantum_host'] = hosts['neutron_host']

        missing = []
        for req in REQUIRED_HOSTS:
            req = '%s_host' % req
            if req not in hosts or not hosts[req]:
                missing.append(req)

        if missing:
            raise base.OSEnvInspectionError(
                'Missing required hosts: %s' % missing)
        self._envdata.update(hosts)

    def enabled_services(self):
        # legacy devstack stuff, but still used elsewhere
        enabled_services = []
        if self._envdata['nova_host']:
            enabled_services += ['n-api', 'n-crt', 'n-obj', 'n-sch']

        if self._envdata['glance_host']:
            enabled_services += ['g-api', 'g-reg']

        if self._envdata['keystone_host']:
            enabled_services += ['key']

        if self._envdata['nova_volume_host']:
            enabled_services += ['n-vol']

        if self._envdata['ceph_host']:
            enabled_services += ['ceph']

        if self._envdata['cinder_host']:
            enabled_services += ['cinder,c-api,c-vol']

        if self._envdata['horizon_host']:
            enabled_services += ['horizon']

        if self._envdata['quantum_host']:
            enabled_services += ['quantum']
        elif self._envdata['neutron_host']:
            enabled_services += ['quantum', 'neutron']
        else:
            enabled_services += ['n-net']

        self._envdata['enabled_services'] = ','.join(enabled_services)

        if self._envdata['swift_host']:
            self._envdata['swift_available'] = 'true'
        else:
            self._envdata['swift_available'] = 'false'

        if 'quantum' in enabled_services:
            self._envdata['quantum_available'] = 'true'
            self._envdata['neutron_available'] = 'true'

        return self._envdata

    def do_inspection(self, data=None):
        self.find_hosts()
        self.enabled_services()
        self.os_admin_creds()
        self.ec2_admin_creds()
        return self._envdata
