import subprocess
import logging
import os

class SchrootNotFound(Exception):
    '''Schroot not found.'''
    def __init__(self, message=None, **kwargs):
        logging.error(message)

class SchrootInvalid(Exception):
    '''Schroot not found.'''
    def __init__(self, message=None, **kwargs):
        logging.error(message)


class SchrootExecutor(object):
    def __init__(self, release=None,
                 ubuntu_release = None,
                 openstack_release=None,
                 base_packages=None):
        def _locate_schroot():
            cmd = [ 'which', 'schroot' ]
            try:
                res = subprocess.check_output(cmd)
            except subprocess.CalledProcessError:
                raise SchrootNotFound('Could not locate schroot binary in $PATH.')

            return res.strip()

        self.schroot_bin = _locate_schroot()
        self.releases = []
        self.schroot = None
        self.session = None
        self.packages = base_packages
        cmd = [ self.schroot_bin, '-l' ]
        res = subprocess.check_output(cmd)
        res = res.split('\n')
        self.releases = []
        for rel in res:
            if rel[:6] == 'source':
                self.releases.append(rel[7:])

        if release:
            self.set_release(release)

        self.openstack_release = openstack_release
        self.ubuntu_release = ubuntu_release

        self.target = "{}-{}".format(ubuntu_release,
                                     openstack_release)
        logging.info('Creating schroot for {}'.format(self.target))

    def set_release(self, release):
        if release not in self.releases:
            raise SchrootInvalid('Invalid schoot. Available schroots: %s' %
                                  self.releases)
        self.release = release

    def execute(self, command, user=None):
        if not user:
            user = os.environ['USER']
        cmd = [ self.schroot_bin, '-p', '-r', '-c', self.session,
                '-u', user, '--' ] + command
        subprocess.check_call(cmd)


    def initialize_schroot(self):
        if not self.release:
            raise SchrootInvalid('No schroot set.')
        cmd = [ self.schroot_bin, '-b', '-c', self.release ]
        self.session = subprocess.check_output(cmd).strip()

        self.execute(['apt-get', '-qy', 'install',
                      'software-properties-common'],
                      user='root')
        self.execute(
            ['add-apt-repository', '-y',
             'ppa:openstack-ubuntu-testing/'
             '{}'.format(self.openstack_release)],
             user='root'
        )

        self.execute(['apt-get', 'update', '-q'],
                     user='root')

        if self.packages:
            cmd = [ 'apt-get', '-y', 'install' ] + self.packages
            self.execute(command=cmd, user='root')

    def cleanup_schroot(self):
        if self.session:
            cmd = [ self.schroot_bin, '-e', '-c', self.session ]
            subprocess.check_call(cmd)
