import os
import unittest
import logging

from openstack_ubuntu_testing.test_utils.juju import client
from openstack_ubuntu_testing.test_utils.juju_deployer import DeployerTest

from tests.charm_test.config import OSCharmTestConfig


class OSCharmTestRunner(unittest.TestCase, DeployerTest):
    @classmethod
    def setUpClass(self):
        self.config = OSCharmTestConfig()

        logging.info('Destroying any existing environment.')
        client.call(['destroy-environment', '-y', '-e',
                    self.config.get('ubuntu_release')])

        env = self.config.get('ubuntu_release')
        constraints = self.config.get('constraints')
        logging.info(
            'Bootstrapping environment %s with constraints (%s).' %
            (env, constraints)
        )
        client.ensure_bootstrap(environment=env, constraints=constraints)

    def test_001_deploy(self):
        self.deploy()

    def test_002_collect_artifacts(self):
        # store artifacts in current jenkins workspace.
        outdir = os.path.join(os.environ.get('WORKSPACE'), 'artifacts')
        self.collect_artifacts(self.config.get('ubuntu_release'), outdir)
