#!/usr/bin/python
import ConfigParser
import os
import uuid
import pprint
import yaml
import logging
import subprocess

from openstack_ubuntu_testing.test_utils.config import CloudConfig
cloud_config = CloudConfig()

DEFAULT_CONFIG='etc/tests/juju.conf'

def ec2_env():
    env = {
            'type': 'ec2',
            'control-bucket': 'test-bucket-' + uuid.uuid4().hex,
            'admin-secret':  uuid.uuid4().hex,
            'access-key': cloud_config.get('ec2', 'access_key'),
            'secret-key': cloud_config.get('ec2', 'secret_key'),
            'ec2-uri': cloud_config.get('ec2', 'ec2_url'),
            's3-uri': cloud_config.get('ec2', 's3_url'),
            'default-image-id': cloud_config.get('ec2', 'ec2_ami_id'),
            'default-series': 'precise'
          }
    return env;


class JujuTestConfig(object):
    def __init__(self, conf=None, home_dir=os.getenv('HOME')):
        if not conf:
            conf = DEFAULT_CONFIG

        if not home_dir:
            raise Exception('No home directory.')
        self.home_dir = home_dir
        self.config = None
        self.providers = []

        # Test at least the EC2 provider, but let others
        # be specified in config.
        if os.path.isfile(conf):
            self.config = ConfigParser.SafeConfigParser()
            self.config.read(conf)
            to_test = self.config.get('DEFAULT', 'providers')
            for p in to_test.split(','):
                self.providers.append(p.strip())
        else:
            self.providers = [ 'ec2' ]

    def get(self, item):
        return self.config.get('DEFAULT', item)

    def get_env(self, type):
        if type == 'ec2':
            return ec2_env()

    def generate_environment_config(self, directory=None):
        '''Write out environments.yaml to directory.
           Create directory if it doesn't exist.'''

        if not directory:
            directory = os.path.join(self.home_dir, '.juju/')

        if not os.path.exists(directory):
            os.makedirs(directory)

        if os.path.isfile(os.path.join(directory, 'environments.yaml')):
            print "Env already exist"
            return

        env = { 'juju': 'environments',
               'environments': {}}

        for e in self.providers:
            name = '%s-test-env-%s' % (e, uuid.uuid4())
            env['environments'][name] = {}
            for k, v in self.get_env(type=e).iteritems():
               env['environments'][name][k] = v
            env['environments'][name] = self.get_env(type=e)

        f = open(os.path.join(directory, 'environments.yaml'), 'w+')
        f.write(yaml.dump(env, default_flow_style=False))
        f.close()

    def generate_ssh_keypair(self, directory=None):
        if not directory:
            directory = os.path.join(self.home_dir, '.ssh')
        if not os.path.exists(directory):
            os.makedirs(directory)

        priv_key = os.path.join(directory, 'id_rsa')
        pub_key = os.path.join(directory, 'id_rsa.pub')

        if not os.path.isfile(priv_key):
            logging.debug('Generating new ssh private key @ %s.' % priv_key)
            cmd = ['ssh-keygen', '-f', priv_key,
                    '-N', '', '-t', 'rsa', '-q']
            subprocess.check_call(cmd)

        if not os.path.isfile(pub_key):
            logging.debug('Generating new ssh public key @ %s.' % pub_key)
            cmd = ['ssh-keygen', '-y', '-f', priv_key]
            key = subprocess.check_output(cmd)
            out = open(pub_key, 'w+')
            out.write(key)
            out.close()

        ssh_config = os.path.join(directory, 'config')
        if not os.path.isfile(ssh_config):
            # Disable host key checking, as well
            config = open(ssh_config, 'w+')
            conf = "Host *\n"\
                   "StrictHostKeyChecking no\n"\
                   "CheckHostIP no\n"\
                   "UserKnownHostsFile /dev/null\n"
            config.write(conf)
            config.close()
