import logging
import os
import unittest

from deployer.utils import setup_logging, ErrorExit

from openstack_ubuntu_testing.test_utils.juju.juju_deployer import DeployerTest

from tests.openstack_deploy.config import OSDeploymentTestConfig

setup_logging(verbose=True, debug=True)


class OSDeploymentTest(unittest.TestCase, DeployerTest):
    @classmethod
    def setUpClass(self):
        self.config = OSDeploymentTestConfig()

        # Jenkins jobs pass installation source to deploy from as
        # INSTALL_SOURCE. Translate it to a deployer override.
        if self.config.get('install_source') not in ['default', None]:
            self.config.set(
                'overrides',
                ['openstack-origin=%s' % self.config.get('install_source')]
            )

    def test_001_deploy(self):
        # Run deployment. Jenkins job can specify name of target
        # juju env in its environment.
        logging.info('Kicking off deployment.')
        try:
            self.deploy(self.config.get('juju_environment'))
        except ErrorExit as e:
            self.fail('Deployer failed: %s' % e)

    def test_002_collect_artifacts(self):
        logging.info('Collecting log artifcacts from deployment.')
        outdir = os.path.join(self.config.get('workspace'), 'artifacts')
        self.collect_artifacts(self.config.get('juju_environment'), outdir)
